package fr.uca.iut.services;

import com.mongodb.lang.Nullable;
import fr.uca.iut.entities.Move;
import fr.uca.iut.entities.Pokemong;
import fr.uca.iut.repositories.MoveRepository;
import fr.uca.iut.utils.StringUtils;
import fr.uca.iut.utils.exceptions.NonValidEntityException;
import jakarta.annotation.PostConstruct;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import org.jetbrains.annotations.NotNull;

import java.util.List;

@ApplicationScoped
public class MoveService extends GenericService<Move> {

    @Inject
    MoveRepository moveRepository;

    @PostConstruct
    public void init() {
        setRepository(moveRepository);
    }

    @Inject
    PokemongService pokemongService;

    @Override
    public void deleteOneById(String id) {
        super.deleteOneById(id);
        // FIXME the deleted move does not get its PokemongMove deleted from any Pokemong's moveset in DB after a move is deleted
        List<Pokemong> pokemongs = pokemongService.findByMove(id);
        for (Pokemong pokemong : pokemongs) {
            pokemong.removeMove(id);
            pokemongService.updateOne(pokemong);
        }
    }

    @Override
    @Nullable
    public Move updateOne(@NotNull Move move) {
        Move existingMove = moveRepository.findById(move.getId());
        if (existingMove != null) {
            existingMove.setName(move.getName());
            // FIXME the updated name does not appear in DB after a move is updated
            List<Pokemong> pokemongs = pokemongService.findByMove(move.getId());
            for (Pokemong pokemong : pokemongs) {
                pokemong.updateMove(move.getId(), move.getName());
                pokemongService.updateOne(pokemong);
            }

            existingMove.setPower(move.getPower());
            existingMove.setCategory(move.getCategory());
            existingMove.setAccuracy(move.getAccuracy());
            existingMove.setType(move.getType());
            moveRepository.persistOrUpdate(existingMove);
        }
        return existingMove;
    }

    @Override
    public void validateOne(Move move) {

        super.validateOne(move);

        if (StringUtils.isBlankString(move.getName())) {
            throw new NonValidEntityException("move name was null, blank or empty");
        }

        if (move.getPower() == null || move.getPower() < 0) {
            throw new NonValidEntityException("move power was null or negative");
        }

        if (move.getCategory() == null) {
            throw new NonValidEntityException("move category was null or invalid");
        }

        if (move.getAccuracy() == null || move.getAccuracy() < 0) {
            throw new NonValidEntityException("move accuracy was null or negative");
        }

        if (move.getType() == null) {
            throw new NonValidEntityException("move type was null or invalid");
        }
    }
}
