package fr.uca.iut.services;

import com.mongodb.lang.Nullable;
import fr.uca.iut.entities.Pokemong;
import fr.uca.iut.entities.PokemongMove;
import fr.uca.iut.entities.Type;
import fr.uca.iut.repositories.MoveRepository;
import fr.uca.iut.repositories.PokemongRepository;
import fr.uca.iut.utils.exceptions.NonValidEntityException;
import jakarta.annotation.PostConstruct;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Set;

@ApplicationScoped
public class PokemongService extends GenericService<Pokemong> {

    @Inject
    PokemongRepository pokemongRepository;
    @Inject
    MoveRepository moveRepository;

    @PostConstruct
    public void init() {
        setRepository(pokemongRepository);
    }

    @Override
    @Nullable
    public Pokemong updateOne(@NotNull Pokemong pokemong) {
        Pokemong existingPokemong = pokemongRepository.findById(pokemong.getId());
        if (existingPokemong != null) {
            existingPokemong.setNickname(pokemong.getNickname());
            existingPokemong.setDob(pokemong.getDob());
            existingPokemong.setLevel(pokemong.getLevel());
            existingPokemong.setPokedexId(pokemong.getPokedexId());
            existingPokemong.setEvoStage(pokemong.getEvoStage());
            existingPokemong.setEvoTrack(pokemong.getEvoTrack());
            existingPokemong.setTrainer(pokemong.getTrainer());
            existingPokemong.setTypes(pokemong.getTypes());
            existingPokemong.setMoveSet(pokemong.getMoveSet());
            pokemongRepository.persistOrUpdate(existingPokemong);
        }
        return existingPokemong;
    }

    public List<Pokemong> findByMove(String id) {
        return pokemongRepository.findByMove(id);
    }

    @Override
    public void validateOne(Pokemong pokemong) {

        super.validateOne(pokemong);

        if (pokemong.getDob() == null) {
            throw new NonValidEntityException("pokemong date of birth was null or invalid");
        }

        if (pokemong.getLevel() == null || pokemong.getLevel() < 1) {
            throw new NonValidEntityException("pokemong level was null or less than 1");
        }

        if (pokemong.getPokedexId() == null || pokemong.getPokedexId() < 1) {
            throw new NonValidEntityException("pokemong pokedex id was null or less than 1");
        }

        if (pokemong.getEvoStage() == null || pokemong.getEvoStage() < 0) {
            throw new NonValidEntityException("pokemong evo stage was null or negative");
        }

        if (pokemong.getEvoTrack() == null) {
            throw new NonValidEntityException("pokemong evo track was null or invalid");
        }

        List<Type> types = pokemong.getTypes();
        if (types == null
            || types.size() == 0
            || types.size() > 2)
        {
            throw new NonValidEntityException("pokemong types was null or empty or had more than 2 types");
        }

        Set<PokemongMove> moveSet = pokemong.getMoveSet();
        if (moveSet == null
            || moveSet.size() == 0
            || moveSet.size() > 4)
        {
            throw new NonValidEntityException("pokemong move set was null or empty or had more than 4 moves");
        }

        for (PokemongMove move : moveSet) {
            String moveId = move.getId();
            if (moveId == null || !moveRepository.existsById(moveId)) {
                throw new NonValidEntityException("move with id " + moveId + " does not exist");
            }
        }
    }
}
